import ollama_python
import json
from util import print_err

MAX_ATTEMPTS = 2

class SchemaValidationError(Exception):
    pass


response = ollama.chat(model='llama3.1', messages=[
  {
    'role': 'user',
    'content': 'Why is the sky blue?',
  },
])
print(response['message']['content'])
async def system_prompt(prompt, input, model, temp=1):
    messages = [
        {"role": "system", "content": prompt},
        {"role": "user", "content": input},
    ]
    response = await generate_chat_response(messages, model, temp)
    return get_message_text(response), get_total_cost([response])

async def json_prompt(
    prompt_text,
    input,
    method,
    model,
    schema=None,
    attempt=1,
    best=(0, None),
    temp=1,
):
    response_txt, cost = await method(prompt_text, input, model, temp)
    response_txt = response_txt.replace("```json", "").replace("```", "").strip()
    try:
        serialized = json.loads(response_txt)
    except json.decoder.JSONDecodeError as e:
        if attempt < MAX_ATTEMPTS:
            return await json_prompt(
                prompt_text, input, method, model, schema, attempt + 1, best, temp
            )
        else:
            raise e
    # Schema is a method that returns a value between 0 and 1
    # 1 indicates full compliance with the schema
    # 0 indicates non compliance with the schema
    # In between values indicate partial compliance
    # The code will retry until the schema check returns 1, saving the "best so far"
    # If MAX ATTEMPTS is reached, then we return the "best so far"
    # If it's None then raise an exception
    # TBD if we should return None instead, could be useful either way
    if schema:
        check = schema(serialized)
        if check != 1:
            if attempt < MAX_ATTEMPTS:
                print_err(f"Schema check failed, retrying ({attempt}) ({serialized})")
                if check > best[0]:
                    best = (check, serialized)
                return await json_prompt(
                    prompt_text, input, method, model, schema, attempt + 1, best
                )
            else:
                if best[1]:
                    return best[1]
                else:
                    raise SchemaValidationError
    return serialized, cost

async def json_prompt_system(
    prompt_text, input, model, schema=None, temp=1
):
    return await json_prompt(
        prompt_text, input, method=system_prompt, model=model, schema=schema, temp=temp
    )