import undetected_chromedriver as uc
from selenium.webdriver.common.by import By
from selenium.common.exceptions import NoSuchElementException
import time
import random
from .util import print_err

# Sleep for a random amount of time 
# in uniform interval 0.5*base and 1.5*base
def sleep_rnd(base=5):
        time.sleep((random.random() * base + base) * 0.5)

class WebScraper:
    driver = None
    delay = 2
    
    def nav(self, url):
        print_err(url)
        if self.driver.current_url != url:
            self.driver.get(url)
            sleep_rnd(self.delay)


    def find_element(self, xpath):
        try:
            return self.driver.find_element(By.XPATH, xpath)
        except NoSuchElementException:
            return None

    def find_elements(self, xpath):
        return self.driver.find_elements(By.XPATH, xpath)

    def find_element_text(self, xpath):
        ele = self.find_element(xpath)
        if ele:
            return ele.text
        else:
            return None

    def find_element_attr(self, xpath, attr):
        ele = self.find_element(xpath)
        if ele:
            return ele.get_attribute(attr)
        else:
            return None
    
    def close(self):
        self.driver.quit()


    def __init__(self, delay=2):
        chrome_options = uc.ChromeOptions()
        chrome_options.headless = False

        oses = [
            "Windows NT 10.0; Win64; x64",
            "Macintosh; Intel Mac OS X 10_15_7",
            "Intel Mac OS X 10.15; rv:133.0"
        ]
        browsers = [
            "Firefox/133.0",
            "Chrome/119.0.0.0",
            "Chrome/120.0.0.0",
            "Chrome/121.0.0.0",
            "Chrome/122.0.0.0",
            "Chrome/123.0.0.0",
            "Chrome/124.0.0.0",
            "Chrome/125.0.0.0",
            "Chrome/126.0.0.0",
        ]
        os = random.choice(oses)
        browser = random.choice(browsers)
        user_agent = f'Mozilla/5.0 ({os}) AppleWebKit/537.36 (KHTML, like Gecko) {browser} Safari/537.36'
        chrome_options.add_argument(f"--user-agent={user_agent}")
        self.driver = uc.Chrome(options = chrome_options)
        self.delay = delay

from playwright.sync_api import sync_playwright
from playwright_stealth import stealth_sync



class PlaywrightScraper:
    driver = None
    page = None
    delay = 2
    
    def nav(self, url):
        print_err(url)
        #if self.driver.current_url != url:
        #    self.driver.get(url)
        #    
        self.page = self.driver.new_page()
        stealth_sync(self.page)
        self.page.goto(url)
        sleep_rnd(self.delay)

    def find_element(self, xpath):
        return self.page.query_selector(xpath)

    def find_elements(self, xpath):
        return self.page.query_selector_all(xpath)

    def find_element_text(self, xpath):
        ele = self.find_element(xpath)
        if ele:
            return ele.text_content()
        else:
            return None

    def find_element_attr(self, xpath, attr):
        ele = self.find_element(xpath)
        if ele:
            return ele.get_attribute(attr)
        else:
            return None
    
    def close(self):
        self.driver.close()


    def __init__(self, delay=2):
        p = sync_playwright().start()
        self.driver = p.chromium.launch()
        self.delay = delay



